/**
 * Copyright © 2014-2019 Darwino, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.darwino.domino.napi;

/**
 * This class is a variant of {@link DominoException} that is thrown by methods
 * that don't allow for arbitrary exceptions in their signatures.
 * 
 * @author Jesse Gallagher
 * @since 2.5.0
 */
public class DominoRuntimeException extends RuntimeException {
	private static final long serialVersionUID = 1L;

	public static DominoRuntimeException forDominoException(DominoException e) {
		DominoRuntimeException result = new DominoRuntimeException(e.getMessage(), e.getStatus());
		result.setStackTrace(e.getStackTrace());
		return result;
	}
	
	private int status = -1;

	private DominoRuntimeException(String message, int status) {
		super(message);
		this.status = status;
	}
    
    /**
     * @return the underlying Domino API status code. See ERR_ in the API documentation for specific
     * 	codes.
     */
    public int getStatus() {
		return status;
	}
	
}
