/**
 * Copyright © 2014-2019 Darwino, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.darwino.domino.napi.enums;

import com.darwino.domino.napi.DominoAPI;
import com.darwino.domino.napi.struct.cd.BaseCDStruct;
import com.darwino.domino.napi.struct.cd.CDBEGINRECORD;
import com.darwino.domino.napi.struct.cd.CDDATAFLAGS;
import com.darwino.domino.napi.struct.cd.CDEMBEDDEDCTL;
import com.darwino.domino.napi.struct.cd.CDENDRECORD;
import com.darwino.domino.napi.struct.cd.CDEXTFIELD;
import com.darwino.domino.napi.struct.cd.CDFIELD;
import com.darwino.domino.napi.struct.cd.CDFIELDHINT;
import com.darwino.domino.napi.struct.cd.CDFILEHEADER;
import com.darwino.domino.napi.struct.cd.CDFILESEGMENT;
import com.darwino.domino.napi.struct.cd.CDGRAPHIC;
import com.darwino.domino.napi.struct.cd.CDHOTSPOTBEGIN;
import com.darwino.domino.napi.struct.cd.CDHeader;
import com.darwino.domino.napi.struct.cd.CDKEYWORD;
import com.darwino.domino.napi.struct.cd.CDTEXT;

/**
 * (ods.h)
 * 
 * @author Jesse Gallagher
 * @since 0.8.0
 *
 */
public enum SIG_CD implements INumberEnum<Short> {
	PDEF_MAIN(DominoAPI.SIG_CD_PDEF_MAIN),
	PDEF_TYPE(DominoAPI.SIG_CD_PDEF_TYPE),
	PDEF_PROPERTY(DominoAPI.SIG_CD_PDEF_PROPERTY),
	PDEF_ACTION(DominoAPI.SIG_CD_PDEF_ACTION),
	TABLECELL_DATAFLAGS(DominoAPI.SIG_CD_TABLECELL_DATAFLAGS), // Note: CDTABLECELLDATAFLAGS is a #define'd alias in the API for CDDATAFLAGS
	EMBEDDEDCONTACTLIST(DominoAPI.SIG_CD_EMBEDDEDCONTACTLIST),
	IGNORE(DominoAPI.SIG_CD_IGNORE),
	TABLECELL_HREF2(DominoAPI.SIG_CD_TABLECELL_HREF2), // Note: this structure is shared among several records
	HREFBORDER(DominoAPI.SIG_CD_HREFBORDER),
	TABLEDATAEXTENSION(DominoAPI.SIG_CD_TABLEDATAEXTENSION),
	EMBEDDEDCALCTL(DominoAPI.SIG_CD_EMBEDDEDCALCTL),
	ACTIONEXT(DominoAPI.SIG_CD_ACTIONEXT),
	EVENT_LANGUAGE_ENTRY(DominoAPI.SIG_CD_EVENT_LANGUAGE_ENTRY),
	FILESEGMENT(DominoAPI.SIG_CD_FILESEGMENT, CDFILESEGMENT.class),
	FILEHEADER(DominoAPI.SIG_CD_FILEHEADER, CDFILEHEADER.class),
	DATAFLAGS(DominoAPI.SIG_CD_DATAFLAGS, CDDATAFLAGS.class),
	BACKGROUNDPROPERTIES(DominoAPI.SIG_CD_BACKGROUNDPROPERTIES),
	EMBEDEXTRA_INFO(DominoAPI.SIG_CD_EMBEDEXTRA_INFO),
	CLIENT_BLOBPART(DominoAPI.SIG_CD_CLIENT_BLOBPART), // Note: this structure is shared with BLOBPART
	CLIENT_EVENT(DominoAPI.SIG_CD_CLIENT_EVENT), // Note: this structure is shared with EVENT
	BORDERINFO_HS(DominoAPI.SIG_CD_BORDERINFO_HS),
	LARGE_PARAGRAPH(DominoAPI.SIG_CD_LARGE_PARAGRAPH),
	EXT_EMBEDDEDSCHED(DominoAPI.SIG_CD_EXT_EMBEDDEDSCHED),
	BOXSIZE(DominoAPI.SIG_CD_BOXSIZE),
	POSITIONING(DominoAPI.SIG_CD_POSITIONING),
	LAYER(DominoAPI.SIG_CD_LAYER),
	DECSFIELD(DominoAPI.SIG_CD_DECSFIELD),
	SPAN_END(DominoAPI.SIG_CD_SPAN_END),
	SPAN_BEGIN(DominoAPI.SIG_CD_SPAN_BEGIN),
	TEXTPROPERTIESTABLE(DominoAPI.SIG_CD_TEXTPROPERTIESTABLE),
	HREF2(DominoAPI.SIG_CD_HREF2), // Note: this structure is shared among several records
	BACKGROUNDCOLOR(DominoAPI.SIG_CD_BACKGROUNDCOLOR), // Note: this structure is shared with COLOR
	INLINE(DominoAPI.SIG_CD_INLINE), V6HOTSPOTBEGIN_CONTINUATION(DominoAPI.SIG_CD_V6HOTSPOTBEGIN_CONTINUATION), TARGET_DBLCLK(DominoAPI.SIG_CD_TARGET_DBLCLK),
	CAPTION(DominoAPI.SIG_CD_CAPTION), LINKCOLORS(DominoAPI.SIG_CD_LINKCOLORS), TABLECELL_HREF(DominoAPI.SIG_CD_TABLECELL_HREF),
	ACTIONBAREXT(DominoAPI.SIG_CD_ACTIONBAREXT),
	IDNAME(DominoAPI.SIG_CD_IDNAME),
	TABLECELL_IDNAME(DominoAPI.SIG_CD_TABLECELL_IDNAME),
	IMAGESEGMENT(DominoAPI.SIG_CD_IMAGESEGMENT),
	IMAGEHEADER(DominoAPI.SIG_CD_IMAGEHEADER),
	V5HOTSPOTBEGIN(DominoAPI.SIG_CD_V5HOTSPOTBEGIN),
	V5HOTSPOTEND(DominoAPI.SIG_CD_V5HOTSPOTEND),
	TEXTPROPERTY(DominoAPI.SIG_CD_TEXTPROPERTY),
	PARAGRAPH(DominoAPI.SIG_CD_PARAGRAPH),
	PABDEFINITION(DominoAPI.SIG_CD_PABDEFINITION),
	PABREFERENCE(DominoAPI.SIG_CD_PABREFERENCE),
	TEXT(DominoAPI.SIG_CD_TEXT, CDTEXT.class),
	HEADER(DominoAPI.SIG_CD_HEADER),
	LINKEXPORT2(DominoAPI.SIG_CD_LINKEXPORT2),
	BITMAPHEADER(DominoAPI.SIG_CD_BITMAPHEADER),
	BITMAPSEGMENT(DominoAPI.SIG_CD_BITMAPSEGMENT),
	COLORTABLE(DominoAPI.SIG_CD_COLORTABLE),
	GRAPHIC(DominoAPI.SIG_CD_GRAPHIC, CDGRAPHIC.class),
//	PMMETASEC(DominoAPI.SIG_CD_PMMETASEC),
	WINMETASEG(DominoAPI.SIG_CD_WINMETASEG),
	MACMETASEG(DominoAPI.SIG_CD_MACMETASEG),
//	CGMETA(DominoAPI.SIG_CD_CGMETA),
	PMMETAHEADER(DominoAPI.SIG_CD_PMMETAHEADER),
	WINMETAHEADER(DominoAPI.SIG_CD_WINMETAHEADER),
	MACMETAHEADER(DominoAPI.SIG_CD_MACMETAHEADER),
	TABLEBEGIN(DominoAPI.SIG_CD_TABLEBEGIN),
	TABLECELL(DominoAPI.SIG_CD_TABLECELL),
	TABLEEND(DominoAPI.SIG_CD_TABLEEND),
	STYLENAME(DominoAPI.SIG_CD_STYLENAME),
	STORAGELINK(DominoAPI.SIG_CD_STORAGELINK),
	TRANSPARENTTABLE(DominoAPI.SIG_CD_TRANSPARENTTABLE),
	HORIZONTALRULE(DominoAPI.SIG_CD_HORIZONTALRULE),
	ALTTEXT(DominoAPI.SIG_CD_ALTTEXT),
	ANCHOR(DominoAPI.SIG_CD_ANCHOR),
	HTMLBEGIN(DominoAPI.SIG_CD_HTMLBEGIN),
	HTMLEND(DominoAPI.SIG_CD_HTMLEND),
	HTMLFORMULA(DominoAPI.SIG_CD_HTMLFORMULA),
	NESTEDTABLEBEGIN(DominoAPI.SIG_CD_NESTEDTABLEBEGIN),
	NESTEDTABLECELL(DominoAPI.SIG_CD_NESTEDTABLECELL),
	NESTEDTABLEEND(DominoAPI.SIG_CD_NESTEDTABLEEND),
	COLOR(DominoAPI.SIG_CD_COLOR),
	TABLECELL_COLOR(DominoAPI.SIG_CD_TABLECELL_COLOR), // Note: this structure is shared with COLOR
	BLOBPART(DominoAPI.SIG_CD_BLOBPART),
	BEGIN(DominoAPI.SIG_CD_BEGIN, CDBEGINRECORD.class),
	END(DominoAPI.SIG_CD_END, CDENDRECORD.class),
	VERTICALALIGN(DominoAPI.SIG_CD_VERTICALALIGN),
	FLOATPOSITION(DominoAPI.SIG_CD_FLOATPOSITION),
	TIMERINFO(DominoAPI.SIG_CD_TIMERINFO),
	TABLEROWHEIGHT(DominoAPI.SIG_CD_TABLEROWHEIGHT),
	TABLELABEL(DominoAPI.SIG_CD_TABLELABEL),
	BIDI_TEXT(DominoAPI.SIG_CD_BIDI_TEXT),
	BIDI_TEXTEFFECT(DominoAPI.SIG_CD_BIDI_TEXTEFFECT),
	REGIONBEGIN(DominoAPI.SIG_CD_REGIONBEGIN),
	REGIONEND(DominoAPI.SIG_CD_REGIONEND),
	TRANSITION(DominoAPI.SIG_CD_TRANSITION),
	FIELDHINT(DominoAPI.SIG_CD_FIELDHINT, CDFIELDHINT.class),
	PLACEHOLDER(DominoAPI.SIG_CD_PLACEHOLDER),
	EMBEDDEDOUTLINE(DominoAPI.SIG_CD_EMBEDDEDOUTLINE),
	EMBEDDEDVIEW(DominoAPI.SIG_CD_EMBEDDEDVIEW),
	CELLBACKGROUNDDATA(DominoAPI.SIG_CD_CELLBACKGROUNDDATA),
	FRAMESETHEADER(DominoAPI.SIG_CD_FRAMESETHEADER),
	FRAMESET(DominoAPI.SIG_CD_FRAMESET),
	FRAME(DominoAPI.SIG_CD_FRAME),
	TARGET(DominoAPI.SIG_CD_TARGET),
	MAPELEMENT(DominoAPI.SIG_CD_MAPELEMENT),
	AREAELEMENT(DominoAPI.SIG_CD_AREAELEMENT),
	HREF(DominoAPI.SIG_CD_HREF), // Note: this structure is shared among several records
	EMBEDDEDCTL(DominoAPI.SIG_CD_EMBEDDEDCTL, CDEMBEDDEDCTL.class),
	HTML_ALTTEXT(DominoAPI.SIG_CD_HTML_ALTTEXT), EVENT(DominoAPI.SIG_CD_EVENT),
	PRETABLEBEGIN(DominoAPI.SIG_CD_PRETABLEBEGIN), BORDERINFO(DominoAPI.SIG_CD_BORDERINFO), EMBEDDEDSCHEDCTL(DominoAPI.SIG_CD_EMBEDDEDSCHEDCTL),
	EXT2_FIELD(DominoAPI.SIG_CD_EXT2_FIELD), EMBEDDEDEDITCTL(DominoAPI.SIG_CD_EMBEDDEDEDITCTL),

	DOCUMENT_PRE_26(DominoAPI.SIG_CD_DOCUMENT_PRE_26), FIELD_PRE_36(DominoAPI.SIG_CD_FIELD_PRE_36),
	FIELD(DominoAPI.SIG_CD_FIELD, CDFIELD.class),
	DOCUMENT(DominoAPI.SIG_CD_DOCUMENT), METAFILE(DominoAPI.SIG_CD_METAFILE), BITMAP(DominoAPI.SIG_CD_BITMAP),
	FONTTABLE(DominoAPI.SIG_CD_FONTTABLE), LINK(DominoAPI.SIG_CD_LINK), LINKEXPORT(DominoAPI.SIG_CD_LINKEXPORT),
	KEYWORD(DominoAPI.SIG_CD_KEYWORD, CDKEYWORD.class),
	LINK2(DominoAPI.SIG_CD_LINK2), CGM(DominoAPI.SIG_CD_CGM), TIFF(DominoAPI.SIG_CD_TIFF),
	PATTERNTABLE(DominoAPI.SIG_CD_PATTERNTABLE), DDEBEGIN(DominoAPI.SIG_CD_DDEBEGIN), DDEEND(DominoAPI.SIG_CD_DDEEND),
	OLEBEGIN(DominoAPI.SIG_CD_OLEBEGIN), OLEEND(DominoAPI.SIG_CD_OLEEND),
	HOTSPOTBEGIN(DominoAPI.SIG_CD_HOTSPOTBEGIN, CDHOTSPOTBEGIN.class),
	HOTSPOTEND(DominoAPI.SIG_CD_HOTSPOTEND), BUTTON(DominoAPI.SIG_CD_BUTTON), BAR(DominoAPI.SIG_CD_BAR),
	V4HOTSPOTBEGIN(DominoAPI.SIG_CD_V4HOTSPOTBEGIN), V4HOTSPOTEND(DominoAPI.SIG_CD_V4HOTSPOTEND),
	EXT_FIELD(DominoAPI.SIG_CD_EXT_FIELD, CDEXTFIELD.class),
	LSOBJECT(DominoAPI.SIG_CD_LSOBJECT), HTMLHEADER(DominoAPI.SIG_CD_HTMLHEADER), HTMLSEGMENT(DominoAPI.SIG_CD_HTMLSEGMENT), LAYOUT(DominoAPI.SIG_CD_LAYOUT),
	LAYOUTTEXT(DominoAPI.SIG_CD_LAYOUTTEXT), LAYOUTEND(DominoAPI.SIG_CD_LAYOUTEND), LAYOUTFIELD(DominoAPI.SIG_CD_LAYOUTFIELD),
	PABHIDE(DominoAPI.SIG_CD_PABHIDE), PABFORMREF(DominoAPI.SIG_CD_PABFORMREF), ACTIONBAR(DominoAPI.SIG_CD_ACTIONBAR),
	ACTION(DominoAPI.SIG_CD_ACTION), DOCAUTOLAUNCH(DominoAPI.SIG_CD_DOCAUTOLAUNCH),
	LAYOUTGRAPHIC(DominoAPI.SIG_CD_LAYOUTGRAPHIC), OLEOBJINFO(DominoAPI.SIG_CD_OLEOBJINFO),
	LAYOUTBUTTON(DominoAPI.SIG_CD_LAYOUTBUTTON), TEXTEFFECT(DominoAPI.SIG_CD_TEXTEFFECT),

	VMHEADER(DominoAPI.SIG_CD_VMHEADER), VMBITMAP(DominoAPI.SIG_CD_VMBITMAP),
//	BMRECT(DominoAPI.SIG_CD_BMRECT),
	VMPOLYGON_BYTE(DominoAPI.SIG_CD_VMPOLYGON_BYTE),
	VMPOLYLINE_BYTE(DominoAPI.SIG_CD_VMPOLYLINE_BYTE), VMREGION(DominoAPI.SIG_CD_VMREGION), VMACTION(DominoAPI.SIG_CD_VMACTION), VMELLIPSE(DominoAPI.SIG_CD_VMELLIPSE),
	VMRNDRECT(DominoAPI.SIG_CD_VMRNDRECT), VMBUTTON(DominoAPI.SIG_CD_VMBUTTON), VMACTION_2(DominoAPI.SIG_CD_VMACTION_2), VMTEXTBOX(DominoAPI.SIG_CD_VMTEXTBOX),
	VMPOLYGON(DominoAPI.SIG_CD_VMPOLYGON), VMPOLYLINE(DominoAPI.SIG_CD_VMPOLYLINE), VMCIRCLE(DominoAPI.SIG_CD_VMCIRCLE), VMPOLYRGN_BYTE(DominoAPI.SIG_CD_VMPOLYRGN_BYTE),

	ALTERNATEBEGIN(DominoAPI.SIG_CD_ALTERNATEBEGIN), ALTERNATEEND(DominoAPI.SIG_CD_ALTERNATEEND), OLERTMARKER(DominoAPI.SIG_CD_OLERTMARKER);
	
	private final short value;
	private final Class<?> clazz;
	
	private SIG_CD(short value, Class<?> clazz) {
		this.value = value;
		this.clazz = clazz;
	}
	private SIG_CD(short value) {
		this(value, null);
	}

	/* (non-Javadoc)
	 * @see com.darwino.domino.napi.enums.INumberEnum#getValue()
	 */
	@Override
	public Short getValue() {
		return value;
	}
	
	@Override
	public long getLongValue() {
		return value;
	}

	/**
	 * @return the implementing class for the record, or <code>null</code> if none has been implemented
	 * @since 2.0.0
	 */
	@SuppressWarnings("unchecked")
	public Class<BaseCDStruct<? extends CDHeader<? extends Number, ? extends Number>>> getInstanceClass() {
		return (Class<BaseCDStruct<? extends CDHeader<? extends Number, ? extends Number>>>)clazz;
	}
}
